
import sys
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import Qt
import urllib.request
from shutil import which
import os.path
import subprocess
HTMLErr='The requested URL was not found on this server.'

hydrophobic=['GLY','ALA','VAL','LEU','ILE','PRO','PHE','MET','TRP','MSE']

propensity = {
    'ALA':0.00,
    'ARG':0.21,
    'ASN':0.65,
    'ASP':0.69,
    'CYS':0.68,
    'GLU':0.40,
    'GLN':0.39,
    'GLY':1.00,
    'HIS':0.56,
    'HSD':0.66,
    'HSE':0.66,
    'HSP':0.66,
    'ILE':0.41,
    'LEU':0.21,
    'LYS':0.26,
    'MET':0.24,
    'PHE':0.54,
    'PRO':3.16,
    'SER':0.50,
    'THR':0.66,
    'TRP':0.49,
    'TYR':0.53,
    'VAL':0.61,
}

class CrossAlphaDet(QWidget):
    def __init__(self):
        super().__init__()
        self.initUI()
    
    def initUI(self):
        self.head=QLabel(self)
        self.head.setText('Cross-Alpha-Det')
        self.head.move(20,5)
        self.head.setFont(QFont("Serif",20,QFont.Bold))

        self.subhead1=QLabel(self)
        self.subhead1.setText('A software for searching possible cross-α amyloid candidates')
        self.subhead1.move(20,40)
        self.subhead1.setFont(QFont('Sans',12))

        self.lblrcsb1=QLabel(self)
        self.lblrcsb1.setText('Enter RCSB PDB ID:')
        self.lblrcsb1.resize(140,30)
        self.lblrcsb1.move(20,80)

        self.txtrcsb=QLineEdit(self)
        self.txtrcsb.setToolTip('Enter four-character <b>RCSB PDB ID</b>')
        self.txtrcsb.resize(140, 30)
        self.txtrcsb.move(180,80)

        self.btnrcsb=QPushButton(self)
        self.btnrcsb.setText('&Download PDB')
        self.btnrcsb.setToolTip('Download PDB File from RCSB')
        self.btnrcsb.move(330,80)
        self.btnrcsb.resize(120,30)
        self.btnrcsb.clicked.connect(self.dlPdb)

        self.lblrcsb2=QLabel(self)
        self.lblrcsb2.setText('')
        self.lblrcsb2.resize(160,30)
        self.lblrcsb2.move(460,80)

        self.lblstrd1=QLabel(self)
        self.lblstrd1.resize(140,30)
        self.lblstrd1.setText('Enter PDB File Path:')
        self.lblstrd1.move(20,120)

        self.txtstrd=QLineEdit(self)
        self.txtstrd.resize(140,30)
        self.txtstrd.setToolTip('Enter your <b>PDB</b> file location')
        self.txtstrd.move(180,120)

        self.btnstrd=QPushButton(self)
        self.btnstrd.setText('Calculate &Stride')
        self.btnstrd.setToolTip('Calculate stride of the PDB file.')
        self.btnstrd.resize(120,30)
        self.btnstrd.move(330,120)
        self.btnstrd.clicked.connect(self.doStride)

        self.lblstrd2=QLabel(self)
        self.lblstrd2.setText('')
        self.lblstrd2.resize(160,30)
        self.lblstrd2.move(460,120)

        self.lblcadt1=QLabel(self)
        self.lblcadt1.setText('Enter Stride Filename:')
        self.lblcadt1.move(20,160)
        self.lblcadt1.resize(150,30)

        self.txtcadt=QLineEdit(self)
        self.txtcadt.move(180,160)
        self.txtcadt.resize(440,30)
        self.txtcadt.setToolTip('Enter your <b>stride</b> file location')

        self.lblangl1=QLabel(self)
        self.lblangl1.setText('Angle:')
        self.lblangl1.move(20,210)
        self.lblangl1.resize(140,30)

        self.lblangl2=QLabel(self)
        self.lblangl2.setText('95°')
        self.lblangl2.move(280,210)
        self.lblangl2.resize(50,30)

        self.sldangl=QSlider(Qt.Horizontal,self)
        self.sldangl.move(180,210)
        self.sldangl.resize(90,30)
        self.sldangl.setMaximum(360)
        self.sldangl.setMinimum(0)
        self.sldangl.setValue(95)
        self.sldangl.valueChanged.connect(self.slideAngle)
        self.sldangl.setToolTip('Select angle through which <b>hydrophobic</b> residues will be calculated<br/>Default value: 95°')

        self.lblcorr1 = QLabel(self)
        self.lblcorr1.setText('Similarity:')
        self.lblcorr1.move(20, 240)
        self.lblcorr1.resize(140, 30)

        self.lblcorr2 = QLabel(self)
        self.lblcorr2.setText('0.75')
        self.lblcorr2.move(280, 240)
        self.lblcorr2.resize(50,30)

        self.sldcorr = QSlider(Qt.Horizontal, self)
        self.sldcorr.move(180, 240)
        self.sldcorr.resize(90, 30)
        self.sldcorr.setMaximum(1000)
        self.sldcorr.setMinimum(0)
        self.sldcorr.setValue(750)
        self.sldcorr.valueChanged.connect(self.slideCorr)
        self.sldcorr.setToolTip('Select the <b>correlation value</b> for determining the cross-α helices.<br/>Default value: 0.75')

        self.lblrnge1=QLabel(self)
        self.lblrnge1.move(20,270)
        self.lblrnge1.setText("Helix Length:")
        self.lblrnge1.resize(140,30)

        self.sbxrnge1=QSpinBox(self)
        self.sbxrnge1.move(180,270)
        self.sbxrnge1.setValue(12)
        self.sbxrnge1.setMinimum(7)
        self.sbxrnge1.resize(50,30)
        self.sbxrnge1.valueChanged.connect(self.setRangeMin)
        self.sbxrnge1.setToolTip('Set the minimum for the length of allowed helices.<br/>Default Value: 12<br/>Minimum Value: 7')

        self.sbxrnge2 = QSpinBox(self)
        self.sbxrnge2.move(270,270)
        self.sbxrnge2.setValue(29)
        self.sbxrnge2.resize(50, 30)
        self.sbxrnge2.setMinimum(self.sbxrnge1.value())
        self.sbxrnge2.valueChanged.connect(self.setRangeMax)
        self.sbxrnge1.setMaximum(self.sbxrnge2.value())
        self.sbxrnge1.setToolTip('Set the maximum for the length of allowed helices.<br/>Default Value: 29')

        self.lblrnge2=QLabel(self)
        self.lblrnge2.move(240,270)
        self.lblrnge2.resize(30,30)
        self.lblrnge2.setText("to")

        self.chkhdro=QCheckBox(self)
        self.chkhdro.move(20,310)
        self.chkhdro.setChecked(True)
        self.chkhdro.setText('Check for minimum hydrophilic residue?')

        self.btnrest=QPushButton(self)
        self.btnrest.move(20,360)
        self.btnrest.setText('Reset')
        self.btnrest.resize(140,30)
        self.btnrest.clicked.connect(self.reset)
        self.btnrest.setToolTip('Press to <b>reset</b>.')

        self.btnproc = QPushButton(self)
        self.btnproc.move(180, 360)
        self.btnproc.setText('&Proceed')
        self.btnproc.setToolTip('Press to <b>search cross-α amyloid candidates</b>.')
        self.btnproc.resize(140, 30)
        self.btnproc.clicked.connect(self.getHelix)

        self.lblstus= QLabel(self)
        self.lblstus.setFont(QFont('Sans',14))
        self.lblstus.setAlignment(Qt.AlignCenter|Qt.AlignHCenter)
        self.lblstus.resize(600,60)
        self.lblstus.move(20,410)
        self.lblstus.setText('')
        self.lblstus.hide()

        self.lblrslt1 = QLabel(self)
        self.lblrslt1.setFont(QFont('Sans',12))
        self.lblrslt1.setAlignment(Qt.AlignCenter | Qt.AlignHCenter)
        self.lblrslt1.resize(290, 30)
        self.lblrslt1.move(330, 210)
        self.lblrslt1.setText('<b>Results</b>')
        self.lblrslt1.hide()

        self.lblrslt2=QLabel(self)
        self.lblrslt2.setAlignment(Qt.AlignCenter|Qt.AlignRight)
        self.lblrslt2.resize(140,20)
        self.lblrslt2.move(330,250)
        self.lblrslt2.setText('Residues:')
        self.lblrslt2.hide()

        self.lblresl1 = QLabel(self)
        self.lblresl1.resize(140, 20)
        self.lblresl1.move(480, 250)
        self.lblresl1.setText('')
        self.lblresl1.hide()

        self.lblrslt3 = QLabel(self)
        self.lblrslt3.setAlignment(Qt.AlignCenter | Qt.AlignRight)
        self.lblrslt3.resize(140, 20)
        self.lblrslt3.move(330, 270)
        self.lblrslt3.setText('Chain ID:')
        self.lblrslt3.hide()

        self.lblresl2 = QLabel(self)
        self.lblresl2.resize(140, 20)
        self.lblresl2.move(480, 270)
        self.lblresl2.setText('')
        self.lblresl2.hide()

        self.lblrslt4 = QLabel(self)
        self.lblrslt4.setAlignment(Qt.AlignCenter | Qt.AlignRight)
        self.lblrslt4.resize(140, 20)
        self.lblrslt4.move(330, 290)
        self.lblrslt4.setText('Correlation:')
        self.lblrslt4.hide()

        self.lblresl3 = QLabel(self)
        self.lblresl3.resize(140, 20)
        self.lblresl3.move(480, 290)
        self.lblresl3.setText('')
        self.lblresl3.hide()

        self.lblrslt5 = QLabel(self)
        self.lblrslt5.setAlignment(Qt.AlignCenter | Qt.AlignRight)
        self.lblrslt5.resize(140, 20)
        self.lblrslt5.move(330, 310)
        self.lblrslt5.setText('Propensity:')
        self.lblrslt5.hide()

        self.lblresl4 = QLabel(self)
        self.lblresl4.resize(140, 20)
        self.lblresl4.move(480, 310)
        self.lblresl4.setText('')
        self.lblresl4.hide()

        self.btnmovl=QPushButton(self)
        self.btnmovl.resize(40,30)
        self.btnmovl.setFont(QFont('Sans',16))
        self.btnmovl.setText('←')
        self.btnmovl.move(350,360)
        self.btnmovl.setDisabled(True)
        self.btnmovl.clicked.connect(self.moveLeft)
        self.btnmovl.hide()
        self.btnmovl.setToolTip('Previous Helix')

        self.btnmovr=QPushButton(self)
        self.btnmovr.resize(40,30)
        self.btnmovr.setFont(QFont('Sans',16))
        self.btnmovr.setText('→')
        self.btnmovr.move(400,360)
        self.btnmovr.setDisabled(True)
        self.btnmovr.hide()
        self.btnmovr.clicked.connect(self.moveRight)
        self.btnmovr.setToolTip('Next Helix')

        self.btnsave=QPushButton(self)
        self.btnsave.resize(140,30)
        self.btnsave.move(480,360)
        self.btnsave.setText('Save as Text')
        self.btnsave.hide()
        self.btnsave.setToolTip('Save as text file')
        self.btnsave.clicked.connect(self.savefile)

        self.resize(640,480)
        self.setWindowTitle('Cross-Alpha-Det')
        self.setMaximumSize(640,480)
        self.setMinimumSize(640,480)
        self.show()
    
    def savefile(self):
        try:
            savefilename=QFileDialog.getSaveFileName(self,'Save File')
            if savefilename[0] == '':
                return
            fileout=open(savefilename[0],'w')
            for lx in self.listx:
                for pp in lx:
                    fileout.write(str(pp)+'\t')
                fileout.write('\n')
            fileout.close()
            QMessageBox.information(self,'Success', 'File Saved Successfully: '+str(savefilename[0]), QMessageBox.Ok,QMessageBox.Ok)
        except:
            QMessageBox.critical(self, 'Error', 'File Save Error', QMessageBox.Ok, QMessageBox.Ok)

    def moveLeft(self):
        self.listPtr-=1
        self.showlist()

    def moveRight(self):
        self.listPtr+=1
        self.showlist()


    def getHelix(self):
        filestride=self.txtcadt.text()
        if not os.path.isfile(filestride):
            QMessageBox.critical(self,'Error', 'Stride File not found.\nPlease check filename and path', QMessageBox.Ok,QMessageBox.Ok)
            return
        self.txtcadt.setDisabled(True)
        self.txtstrd.setDisabled(True)
        self.txtrcsb.setDisabled(True)
        self.sldcorr.setDisabled(True)
        self.sldangl.setDisabled(True)
        self.sbxrnge1.setDisabled(True)
        self.sbxrnge2.setDisabled(True)
        self.chkhdro.setDisabled(True)
        self.btnrcsb.setDisabled(True)
        self.btnstrd.setDisabled(True)
        self.btnproc.setDisabled(True)

        theta=self.sldangl.value()
        corr=self.sldcorr.value()/1000.0
        helixlw=self.sbxrnge1.value()
        helixup=self.sbxrnge2.value()
        checkmin=self.chkhdro.isChecked()
        self.listx=self.check_possibility(filestride,theta,corr,helixlw,helixup,checkmin)
        self.lblstus.show()
        if len(self.listx) == 0:
            self.lblstus.setText('No cross-α amyloid candidates found')
        else:
            self.lblstus.setText(str(len(self.listx))+' cross-α amyloid candidates found')
            self.btnsave.show()
            self.btnmovl.show()
            self.btnmovr.show()
            self.lblresl4.show()
            self.lblresl3.show()
            self.lblresl2.show()
            self.lblresl1.show()
            self.lblrslt1.show()
            self.lblrslt2.show()
            self.lblrslt3.show()
            self.lblrslt4.show()
            self.lblrslt5.show()
            self.listPtr=0
            self.showlist()

    def showlist(self):
        self.lblresl1.setText(str(self.listx[self.listPtr][0]) + ' to ' + str(self.listx[self.listPtr][1]))
        self.lblresl2.setText(str(self.listx[self.listPtr][3]))
        self.lblresl3.setText(str('%.3f'%self.listx[self.listPtr][6]))
        self.lblresl4.setText(str('%.2f'%self.listx[self.listPtr][7])+' kCal/mol')
        if self.listPtr>0:
            self.btnmovl.setDisabled(False)
        else:
            self.btnmovl.setDisabled(True)
        if self.listPtr+1<len(self.listx):
            self.btnmovr.setDisabled(False)
        else:
            self.btnmovr.setDisabled(True)


    def reset(self):
        self.sldcorr.setValue(750)
        self.slideCorr()
        self.sldangl.setValue(95)
        self.slideAngle()
        self.sbxrnge1.setValue(12)
        self.sbxrnge2.setValue(29)
        self.chkhdro.setChecked(True)
        self.txtcadt.setDisabled(False)
        self.txtstrd.setDisabled(False)
        self.txtrcsb.setDisabled(False)
        self.sldcorr.setDisabled(False)
        self.sldangl.setDisabled(False)
        self.sbxrnge1.setDisabled(False)
        self.sbxrnge2.setDisabled(False)
        self.chkhdro.setDisabled(False)
        self.btnrcsb.setDisabled(False)
        self.btnstrd.setDisabled(False)
        self.btnproc.setDisabled(False)
        self.btnsave.hide()
        self.btnmovl.hide()
        self.btnmovr.hide()
        self.lblresl4.hide()
        self.lblresl3.hide()
        self.lblresl2.hide()
        self.lblresl1.hide()
        self.lblrslt1.hide()
        self.lblrslt2.hide()
        self.lblrslt3.hide()
        self.lblrslt4.hide()
        self.lblrslt5.hide()
        self.lblstus.setText('Reset Complete')

    def setRangeMin(self):
        if self.sbxrnge1.value()>self.sbxrnge2.value():
            self.sbxrnge2.setValue(self.sbxrnge1.value())
        self.sbxrnge2.setMinimum(self.sbxrnge1.value())

    def setRangeMax(self):
        if self.sbxrnge1.value() > self.sbxrnge2.value():
            self.sbxrnge1.setValue(self.sbxrnge2.value())
        self.sbxrnge1.setMaximum(self.sbxrnge2.value())

    def slideAngle(self):
        self.lblangl2.setText(str(self.sldangl.value())+'°')

    def slideCorr(self):
        self.lblcorr2.setText(str(self.sldcorr.value()/1000.0))

    def dlPdb(self):
        rcsb_code=self.txtrcsb.text()
        if len(rcsb_code)!=4:
            QMessageBox.critical(self,'Error','Please enter a valid PDB ID.',QMessageBox.Ok,QMessageBox.Ok)
            return
        err=self.download(rcsb_code)
        if err!=0:
            QMessageBox.critical(self, 'Error', 'Please enter a valid PDB ID or\ncheck your internet connection.', QMessageBox.Ok, QMessageBox.Ok)
            return
        else:
            QMessageBox.information(self,'Success','RCSB PDB Downloaded: '+rcsb_code.upper()+'.pdb',QMessageBox.Ok,QMessageBox.Ok)
            self.lblrcsb2.setText('Downloaded: '+rcsb_code.upper()+'.pdb')
            self.txtstrd.setText(rcsb_code.upper()+'.pdb')

    def doStride(self):
        stridecmd='stride'
        while which(stridecmd) is None:
            QMessageBox.critical(self, 'Error', ('Stride not found\n'
                                                 'Please install it in path or\n'
                                                 'input the exectuable path/command.\n'
                                                 '\n'
                                                 'Stride Website: http://webclu.bio.wzw.tum.de/stride/'), QMessageBox.Ok, QMessageBox.Ok)
            stridecmd,ok=QInputDialog.getText(self,'Input','Enter the path of Stride Executable:')
        pdbFile=self.txtstrd.text()
        if not os.path.isfile(pdbFile):
            QMessageBox.critical(self,'Error','Enter a vaild file name\npreferably with full path',QMessageBox.Ok,QMessageBox.Ok)
            return
        strproc=subprocess.Popen(['/bin/bash', '-i','-c',stridecmd+' '+pdbFile+' > '+pdbFile+'.str'])
        output,err=strproc.communicate()
        if os.path.isfile(pdbFile+'.str') and err is None:
            QMessageBox.information(self,'Success','Stride Calculation Successful: '+pdbFile+'.str',QMessageBox.Ok,QMessageBox.Ok)
            self.lblstrd2.setText('Complete: '+pdbFile+'.str')
            self.txtcadt.setText(pdbFile+'.str')
        else:
            QMessageBox.critical(self,'Error','Error in Stride Calculation,\nPlease check in terminal',QMessageBox.Ok,QMessageBox.Ok)


    def download(self,rcsb):
        try:
            url='https://files.rcsb.org/download/'+rcsb.upper()+'.pdb'
            urllib.request.urlretrieve(url,rcsb.upper()+'.pdb')
            return 0
        except:
            return 1

    def get_combinations(self,n,theta):
        listx = []
        for i in range(360):
            num = ''
            for j in range(n):
                if ((i + j * (720 / 7.2)) % 360) <= theta:
                    num += '1'
                else:
                    num += '0'
            listx.append(num)
        return (set(listx))

    def check_sim(self,str1, list2):
        simlist = []
        lx = len(str1)
        for elem in list2:
            numx = 0.0
            for x in range(lx):
                if str1[x] == elem[x]:
                    numx += (1 / lx)
            simlist.append(numx)
        return (max(simlist))

    def check_possibility(self,filename,theta,correl,helixlw,helixup,checkmin):
        try:
            helix = []
            chain_list = []
            charlist = ''
            prop=[]
            filex = open(filename, 'r')
            for line in filex:
                elem = line.strip().split()
                if elem[0] == 'ASG':
                    if elem[5] == 'H':
                        helix.append(int(elem[3]))
                        chain_list.append(elem[2])
                        if elem[1] in hydrophobic:
                            charlist += '0'
                        else:
                            charlist += '1'
                        prop.append(propensity[elem[1]])
            filex.close()

            if len(helix) == 0:
                return
            charlists = [charlist[0]]
            helices = [[helix[0]]]
            chains = [[chain_list[0]]]
            propx = [prop[0]]

            for x in range(1, len(helix)):
                if (helix[x - 1] + 1) != helix[x] or chain_list[x - 1] != chain_list[x]:
                    helices.append([])
                    charlists.append('')
                    chains.append([])
                    propx.append(0.0)
                helices[-1].append(helix[x])
                charlists[-1] += charlist[x]
                chains[-1].append(chain_list[x])
                propx[-1]+=prop[x]


            listx=[]
            for x in range(len(charlists)):
                if helices[x][-1] - helices[x][0] >= 6:
                    combi = self.get_combinations(len(charlists[x]) - 2,theta)
                    cnt_hel = helices[x][-1] - helices[x][0] + 1
                    similarity = self.check_sim(charlists[x][1:-1], combi)
                    striplen=int((cnt_hel-2)/7.2)
                    if similarity >= correl and helixlw <= cnt_hel <= helixup:
                        if checkmin==False or charlists[x][1:-1].count('1')>=striplen:
                            listx.append([])
                            listx[-1].append(helices[x][0])
                            listx[-1].append(helices[x][-1])
                            listx[-1].append(cnt_hel)
                            listx[-1].append(chains[x][0])
                            listx[-1].append(charlists[x][1:-1].count('0'))
                            listx[-1].append(charlists[x][1:-1].count('1'))
                            listx[-1].append(similarity)
                            listx[-1].append(propx[x])
            return listx
        except:
            QMessageBox.critical(self,'Error','Some Error found while calculation\nPlease check your stride file',QMessageBox.Ok,QMessageBox.Ok)
            return []



if __name__ == '__main__':
    app = QApplication(sys.argv)
    ex = CrossAlphaDet()
    sys.exit(app.exec_())
